@@ -149,9 +149,11 @@ class Client:
              - settings : allows to get index settings (https only)
              - editSettings : allows to change index settings (https only)
     @param validity the number of seconds after which the key will be automatically removed (0 means no time limit for this key)
+    @param maxQueriesPerIPPerHour Specify the maximum number of API calls allowed from an IP address per hour.  Defaults to 0 (no rate limit).
+    @param maxHitsPerQuery Specify the maximum number of hits this API key can retrieve in one call. Defaults to 0 (unlimited) 
     """
-    def addUserKey(self, acls, validity = 0):
-        return AlgoliaUtils_request(self.headers, self.hosts, "POST", "/1/keys", {"acl": acls, "validity": validity} )          
+    def addUserKey(self, acls, validity = 0, maxQueriesPerIPPerHour = 0, maxHitsPerQuery = 0):
+        return AlgoliaUtils_request(self.headers, self.hosts, "POST", "/1/keys", {"acl": acls, "validity": validity, "maxQueriesPerIPPerHour": maxQueriesPerIPPerHour, "maxHitsPerQuery": maxHitsPerQuery} )
 
 """
 Contains all the functions related to one index
@@ -248,41 +250,52 @@ class Index:
     
     @param query the full text query
     @param args (optional) if set, contains an associative array with query parameters:
-      - attributes: a string that contains attribute names to retrieve separated by a comma. 
-        By default all attributes are retrieved.
-      - numerics: specify the list of numeric filters you want to apply separated by a comma. 
-        The syntax of one filter is `attributeName` followed by `operand` followed by `value`. 
-        Supported operands are `<`, `<=`, `=`, `>` and `>=`. 
-        You can have multiple conditions on one attribute like for example `numerics=price>100,price<1000`.
-      - attributesToHighlight: a string that contains attribute names to highlight separated by a comma. 
-        By default indexed attributes are highlighted.
-      - attributesToSnippet: a string that contains the names of attributes to snippet alongside the 
-        number of words to return (syntax is 'attributeName:nbWords'). 
-        Attributes are separated by a comma (Example: "attributesToSnippet=name:10,content:10").
-        By default no snippet is computed.
-      - minWordSizefor1Typo: the minimum number of characters in a query word to accept one typo in this word. 
-        Defaults to 3.
-      - minWordSizefor2Typos: the minimum number of characters in a query word to accept two typos in this word.
-         Defaults to 7.
-      - getRankingInfo: if set to 1, the result hits will contain ranking information in 
-         _rankingInfo attribute
-      - page: (pagination parameter) page to retrieve (zero base). Defaults to 0.
-      - hitsPerPage: (pagination parameter) number of hits per page. Defaults to 10.
-      - aroundLatLng let you search for entries around a given latitude/longitude (two float separated 
-        by a ',' for example aroundLatLng=47.316669,5.016670). 
-        You can specify the maximum distance in meters with aroundRadius parameter (in meters).
-        At indexing, geoloc of an object should be set with _geoloc attribute containing lat and lng attributes (for example {"_geoloc":{"lat":48.853409, "lng":2.348800}})
-      - insideBoundingBox let you search entries inside a given area defined by the two extreme points of 
-        a rectangle (defined by 4 floats: p1Lat,p1Lng,p2Lat, p2Lng.
+      - page: (integer) Pagination parameter used to select the page to retrieve.
+                        Page is zero-based and defaults to 0. Thus, to retrieve the 10th page you need to set page=9
+      - hitsPerPage: (integer) Pagination parameter used to select the number of hits per page. Defaults to 20.
+      - attributesToRetrieve: a string that contains the list of object attributes you want to retrieve (let you minimize the answer size).
+        Attributes are separated with a comma (for example "name,address").
+        You can also use a string array encoding (for example ["name","address"]). 
+        By default, all attributes are retrieved. You can also use '*' to retrieve all values when an attributesToRetrieve setting is specified for your index.
+      - attributesToHighlight: a string that contains the list of attributes you want to highlight according to the query. 
+        Attributes are separated by a comma. You can also use a string array encoding (for example ["name","address"]). 
+        If an attribute has no match for the query, the raw value is returned. By default all indexed text attributes are highlighted. 
+        You can use `*` if you want to highlight all textual attributes. Numerical attributes are not highlighted. 
+        A matchLevel is returned for each highlighted attribute and can contain:
+           - full: if all the query terms were found in the attribute,
+           - partial: if only some of the query terms were found,
+           - none: if none of the query terms were found.
+      - attributesToSnippet: a string that contains the list of attributes to snippet alongside the number of words to return (syntax is `attributeName:nbWords`). 
+         Attributes are separated by a comma (Example: attributesToSnippet=name:10,content:10).
+         You can also use a string array encoding (Example: attributesToSnippet: ["name:10","content:10"]). By default no snippet is computed.
+      - minWordSizefor1Typo: the minimum number of characters in a query word to accept one typo in this word. Defaults to 3.
+      - minWordSizefor2Typos: the minimum number of characters in a query word to accept two typos in this word. Defaults to 7.
+      - getRankingInfo: if set to 1, the result hits will contain ranking information in _rankingInfo attribute.
+      - aroundLatLng: search for entries around a given latitude/longitude (specified as two floats separated by a comma).
+        For example aroundLatLng=47.316669,5.016670). 
+        You can specify the maximum distance in meters with the aroundRadius parameter (in meters) and the precision for ranking with aroundPrecision
+        (for example if you set aroundPrecision=100, two objects that are distant of less than 100m will be considered as identical for "geo" ranking parameter).
+        At indexing, you should specify geoloc of an object with the _geoloc attribute (in the form {"_geoloc":{"lat":48.853409, "lng":2.348800}})
+      - insideBoundingBox: search entries inside a given area defined by the two extreme points of a rectangle (defined by 4 floats: p1Lat,p1Lng,p2Lat,p2Lng).
         For example insideBoundingBox=47.3165,4.9665,47.3424,5.0201).
-        At indexing, geoloc of an object should be set with _geoloc attribute containing lat and lng attributes (for example {"_geoloc":{"lat":48.853409, "lng":2.348800}})
-      - queryType: select how the query words are interpreted:
+        At indexing, you should specify geoloc of an object with the _geoloc attribute (in the form {"_geoloc":{"lat":48.853409, "lng":2.348800}})
+      - numericFilters: a string that contains the list of numeric filters you want to apply separated by a comma. 
+        The syntax of one filter is `attributeName` followed by `operand` followed by `value`. Supported operands are `<`, `<=`, `=`, `>` and `>=`. 
+        You can have multiple conditions on one attribute like for example numerics=price>100,price<1000. 
+        You can also use a string array encoding (for example numericFilters: ["price>100","price<1000"]).
+      - tagFilters: filter the query by a set of tags. You can AND tags by separating them by commas. 
+        To OR tags, you must add parentheses. For example, tags=tag1,(tag2,tag3) means tag1 AND (tag2 OR tag3).
+        You can also use a string array encoding, for example tagFilters: ["tag1",["tag2","tag3"]] means tag1 AND (tag2 OR tag3).
+        At indexing, tags should be added in the _tags** attribute of objects (for example {"_tags":["tag1","tag2"]}). 
+      - facets: filter the query by a list of facets. Facets are separated by commas and each facet is encoded as attributeName:value. 
+        For example: facetFilters=category:Book,author:John%20Doe. 
+        You can also use a string array encoding (for example ["category:Book","author:John%20Doe"]).
+      - queryType: select how the query words are interpreted, it can be one of the following value:
          - prefixAll: all query words are interpreted as prefixes,
          - prefixLast: only the last word is interpreted as a prefix (default behavior),
          - prefixNone: no query word is interpreted as a prefix. This option is not recommended.
-      - tags filter the query by a set of tags. You can AND tags by separating them by commas. To OR tags, you must add parentheses. For example, tags=tag1,(tag2,tag3) means tag1 AND (tag2 OR tag3).
-        At indexing, tags should be added in the _tags attribute of objects (for example {"_tags":["tag1","tag2"]} )
-
+      - optionalWords: a string that contains the list of words that should be considered as optional when found in the query. 
+        The list of words is comma separated.
     """
     def search(self, query, args = None):
         if args == None:
@@ -307,47 +320,55 @@ class Index:
     # Get settings of this index
     def getSettings(self):
         return AlgoliaUtils_request(self.headers, self.hosts, "GET", "/1/indexes/%s/settings" % self.urlIndexName)
+  
+    # This function deletes the index content. Settings and index specific API keys are kept untouched.
+    def clearIndex(self):
+        return AlgoliaUtils_request(self.headers, self.hosts, "POST", "/1/indexes/%s/clear" % self.urlIndexName)
 
     """
       Set settings for this index
       
       @param settigns the settings object that can contains :
-       - minWordSizefor1Typo (integer) the minimum number of characters to accept one typo (default = 3)
-       - minWordSizefor2Typos: (integer) the minimum number of characters to accept two typos (default = 7)
-       - hitsPerPage: (integer) the number of hits per page (default = 10)
-       - attributesToRetrieve: (array of strings) default list of attributes to retrieve for objects
-       - attributesToHighlight: (array of strings) default list of attributes to highlight
-       - attributesToSnippet: (array of strings) default list of attributes to snippet alongside the number of 
-         words to return (syntax is 'attributeName:nbWords').
-         By default no snippet is computed.
-       - attributesToIndex: (array of strings) the list of fields you want to index. 
-         By default all textual and numerical attributes of your objects are indexed, but you should update it to get optimal 
-         results. This parameter has two important uses:
-            - Limit the attributes to index. 
-              For example if you store a binary image in base64, you want to store it in the index but you 
-              don't want to use the base64 string for search.
-            - Control part of the ranking (see the ranking parameter for full explanation). 
-              Matches in attributes at the beginning of the list will be considered more important than matches 
-              in attributes further down the list.
-              In one attribute, matching text at the beginning of the attribute will be considered more important than text after, 
-              you can disable this behavior if you add your attribute inside `unordered(AttributeName)`, 
-              for example `attributesToIndex:["title", "unordered(text)"]`.
-       - ranking: (array of strings) controls the way results are sorted. 
-          We have four available criteria: 
-            - typo (sort according to number of typos), 
-            - geo: (sort according to decreassing distance when performing a geo-location based search),
-            - proximity: sort according to the proximity of query words in hits, 
-            - attribute: sort according to the order of attributes defined by **attributesToIndex**,
-            - exact: sort according to the number of words that are matched identical to query word (and not as a prefix),
-            - custom which is user defined
-          (the standard order is ["typo", "geo", "proximity", "attribute", "exact", "custom"])
-       - queryType: select how the query words are interpreted:
-            - prefixAll: all query words are interpreted as prefixes,
-            - prefixLast: only the last word is interpreted as a prefix (default behavior),
-            - prefixNone: no query word is interpreted as a prefix. This option is not recommended.
-       - customRanking: (array of strings) lets you specify part of the ranking. 
-         The syntax of this condition is an array of strings containing attributes prefixed 
-         by asc (ascending order) or desc (descending order) operator.
+       - minWordSizefor1Typo: (integer) the minimum number of characters to accept one typo (default = 3).
+       - minWordSizefor2Typos: (integer) the minimum number of characters to accept two typos (default = 7).
+       - hitsPerPage: (integer) the number of hits per page (default = 10).
+       - attributesToRetrieve: (array of strings) default list of attributes to retrieve in objects. 
+         If set to null, all attributes are retrieved.
+       - attributesToHighlight: (array of strings) default list of attributes to highlight. 
+         If set to null, all indexed attributes are highlighted.
+       - attributesToSnippet**: (array of strings) default list of attributes to snippet alongside the number of words to return (syntax is attributeName:nbWords).
+         By default no snippet is computed. If set to null, no snippet is computed.
+       - attributesToIndex: (array of strings) the list of fields you want to index.
+         If set to null, all textual and numerical attributes of your objects are indexed, but you should update it to get optimal results.
+         This parameter has two important uses:
+           - Limit the attributes to index: For example if you store a binary image in base64, you want to store it and be able to 
+             retrieve it but you don't want to search in the base64 string.
+           - Control part of the ranking*: (see the ranking parameter for full explanation) Matches in attributes at the beginning of 
+             the list will be considered more important than matches in attributes further down the list. 
+             In one attribute, matching text at the beginning of the attribute will be considered more important than text after, you can disable 
+             this behavior if you add your attribute inside `unordered(AttributeName)`, for example attributesToIndex: ["title", "unordered(text)"].
+       - attributesForFaceting: (array of strings) The list of fields you want to use for faceting. 
+         All strings in the attribute selected for faceting are extracted and added as a facet. If set to null, no attribute is used for faceting.
+       - ranking: (array of strings) controls the way results are sorted.
+         We have six available criteria: 
+          - typo: sort according to number of typos,
+          - geo: sort according to decreassing distance when performing a geo-location based search,
+          - proximity: sort according to the proximity of query words in hits,
+          - attribute: sort according to the order of attributes defined by attributesToIndex,
+          - exact: sort according to the number of words that are matched identical to query word (and not as a prefix),
+          - custom: sort according to a user defined formula set in **customRanking** attribute.
+         The standard order is ["typo", "geo", "proximity", "attribute", "exact", "custom"]
+       - customRanking: (array of strings) lets you specify part of the ranking.
+         The syntax of this condition is an array of strings containing attributes prefixed by asc (ascending order) or desc (descending order) operator.
+         For example `"customRanking" => ["desc(population)", "asc(name)"]`  
+       - queryType: Select how the query words are interpreted, it can be one of the following value:
+         - prefixAll: all query words are interpreted as prefixes,
+         - prefixLast: only the last word is interpreted as a prefix (default behavior),
+         - prefixNone: no query word is interpreted as a prefix. This option is not recommended.
+       - highlightPreTag: (string) Specify the string that is inserted before the highlighted parts in the query result (default to "<em>").
+       - highlightPostTag: (string) Specify the string that is inserted after the highlighted parts in the query result (default to "</em>").
+       - optionalWords: (array of strings) Specify a list of words that should be considered as optional when found in the query.
+ 
      """
     def setSettings(self, settings):
         return AlgoliaUtils_request(self.headers, self.hosts, "PUT", "/1/indexes/%s/settings" % self.urlIndexName, settings)
@@ -378,9 +399,11 @@ class Index:
              - settings : allows to get index settings (https only)
              - editSettings : allows to change index settings (https only)
     @param validity the number of seconds after which the key will be automatically removed (0 means no time limit for this key)
+    @param maxQueriesPerIPPerHour Specify the maximum number of API calls allowed from an IP address per hour.  Defaults to 0 (no rate limit).
+    @param maxHitsPerQuery Specify the maximum number of hits this API key can retrieve in one call. Defaults to 0 (unlimited) 
     """
-    def addUserKey(self, acls, validity = 0):
-        return AlgoliaUtils_request(self.headers, self.hosts, "POST", "/1/indexes/%s/keys" % self.urlIndexName, {"acl": acls, "validity": validity} )   
+    def addUserKey(self, acls, validity = 0, maxQueriesPerIPPerHour = 0, maxHitsPerQuery = 0):
+        return AlgoliaUtils_request(self.headers, self.hosts, "POST", "/1/indexes/%s/keys" % self.urlIndexName, {"acl": acls, "validity": validity, "maxQueriesPerIPPerHour": maxQueriesPerIPPerHour, "maxHitsPerQuery": maxHitsPerQuery} )
 
 # Util function used to send request
 def AlgoliaUtils_request(headers, hosts, method, request, body = None):
